#include <iostream>
#include <vector>
#include <list>
#include <algorithm>
#include <ctime>

//Show how to use namespace operator
using namespace std;


int main(){
  cout << "Let's figure out how to use some data structures." << endl;
  cout << "We'll make a list of jumbled numbers and show how a vector and a linked list will both perform when sorting them." << endl;

  cout<< "First, input the length of our list: ";
  int N;
  cin >> N;
  cout << "OK, we'll start by making a vector of length " << N << "." << endl;

  vector<double> V(N);
  for(int i = 0; i < N; i++){
    V[i] = i+1;
  }

  cout << "Our unjumbled vector is V = [ ";
  for(int i = 0; i < N; i++){
    cout << V[i] << " ";
  }
  cout << "] " << endl;
  
  cout << "We'll sweep through the vector and swap elements." << endl;
  for(int i = 0; i < N; i++){
    //Pick a random number between 0 and N-1
    int j = rand()%N;
    double toswap = V[j];
    V[j] = V[i];
    V[i] = toswap;
  }
  cout << "Our jumbled vector is V = [ ";
  for(int i = 0; i < N; i++){
    cout << V[i] << " ";
  }
  cout << "] " << endl;
  
  cout << "Let's make a linked list with the same elements as V." << endl;
  list<double> L;
  for(int i = 0; i < N; i++){
    L.push_back(V[i]);
  }
  cout << "Our jumbled linked list is L = [ ";
  for(list<double>::iterator it=L.begin(); it!=L.end();it++){
    cout << *it << " ";
  }
  cout << "]" << endl;

  cout << "Now we'll resort our data structures and measure how long it took." << endl;
  
  clock_t Vsort_start, Lsort_start;
  Vsort_start = clock();
  sort(V.begin(),V.end());
  double Vtime = ( clock() - Vsort_start ) / (double) CLOCKS_PER_SEC;
  Lsort_start = clock();
  L.sort();
  double Ltime = ( clock() - Lsort_start ) / (double) CLOCKS_PER_SEC;


  cout << "Our sorted vector is V = [ ";
  for(int i = 0; i < N; i++){
    cout << V[i] << " ";
  }
  cout << "] " << endl;
  cout << "Our sorted linked list is L = [ ";
  for(list<double>::iterator it=L.begin(); it!=L.end();it++){
    cout << *it << " ";
  }
  cout << "]" << endl;

  cout << "Vector sort time: " << Vtime << endl;
  cout << "Linked list sort time: " << Ltime << endl;


  return 0;
}
